@@ -1,19 +1,15 @@
 package DBM::Deep;
 
-use 5.006_000;
+use 5.008_000;
 
 use strict;
 use warnings FATAL => 'all';
 no warnings 'recursion';
 
-our $VERSION = q(1.0019_003);
+our $VERSION = q(1.9999_01);
 
 use Scalar::Util ();
 
-use overload
-    '""' => sub { overload::StrVal( $_[0] ) },
-    fallback => 1;
-
 use constant DEBUG => 0;
 
 use DBM::Deep::Engine;
@@ -21,6 +17,11 @@ use DBM::Deep::Engine;
 sub TYPE_HASH   () { DBM::Deep::Engine->SIG_HASH  }
 sub TYPE_ARRAY  () { DBM::Deep::Engine->SIG_ARRAY }
 
+my %obj_cache; # In external_refs mode, all objects are registered here,
+               # and dealt with in the END block at the bottom.
+use constant HAVE_HUFH => scalar eval{ require Hash::Util::FieldHash };
+HAVE_HUFH and Hash::Util::FieldHash::fieldhash %obj_cache;
+
 # This is used in all the children of this class in their TIE<type> methods.
 sub _get_args {
     my $proto = shift;
@@ -85,9 +86,10 @@ sub _init {
     }, $class;
 
     unless ( exists $args->{engine} ) {
-        my $class = exists $args->{dbi}
-            ? 'DBM::Deep::Engine::DBI'
-            : 'DBM::Deep::Engine::File';
+        my $class =
+            exists $args->{dbi}   ? 'DBM::Deep::Engine::DBI'  :
+            exists $args->{_test} ? 'DBM::Deep::Engine::Test' :
+                                    'DBM::Deep::Engine::File' ;
 
         eval "use $class"; die $@ if $@;
         $args->{engine} = $class->new({
@@ -114,6 +116,22 @@ sub _init {
         die $e;
     }
 
+    if(  $self->{engine}->{external_refs}
+     and my $sector = $self->{engine}->load_sector( $self->{base_offset} )
+    ) {
+        $sector->increment_refcount;
+
+        Scalar::Util::weaken( my $feeble_ref = $self );
+        $obj_cache{ 0+$self } = \$feeble_ref;
+
+        # Make sure this cache is not a memory hog
+        if(!HAVE_HUFH) {
+            for(keys %obj_cache) {
+                delete $obj_cache{$_} if not ${$obj_cache{$_}};
+            }
+        }
+    }
+
     return $self;
 }
 
@@ -134,10 +152,15 @@ sub lock_exclusive {
     return $self->_engine->lock_exclusive( $self, @_ );
 }
 *lock = \&lock_exclusive;
+
 sub lock_shared {
     my $self = shift->_get_self;
-use Carp qw( cluck ); use Data::Dumper;
-cluck Dumper($self) unless $self->_engine;
+    # cluck() the problem with cached File objects.
+    unless ( $self->_engine ) {
+        require Carp;
+        require Data::Dumper;
+        Carp::cluck( Data::Dumper->Dump( [$self], ['self'] ) );
+    }
     return $self->_engine->lock_shared( $self, @_ );
 }
 
@@ -188,14 +211,6 @@ sub _copy_value {
     return 1;
 }
 
-#sub _copy_node {
-#    die "Must be implemented in a child class\n";
-#}
-#
-#sub _repr {
-#    die "Must be implemented in a child class\n";
-#}
-
 sub export {
     my $self = shift->_get_self;
 
@@ -381,6 +396,10 @@ sub supports {
     return $self->_engine->supports( @_ );
 }
 
+sub db_version {
+    shift->_get_self->_engine->db_version;
+}
+
 #XXX Migrate this to the engine, where it really belongs and go through some
 # API - stop poking in the innards of someone else..
 {
@@ -589,7 +608,7 @@ sub CLEAR {
         $engine->clear( $self );
     };
     my $e = $@;
-    warn "$e\n" if $e;
+    warn "$e\n" if $e && DEBUG;
 
     $self->unlock;
 
@@ -609,5 +628,54 @@ sub clear  { (shift)->CLEAR( @_ )  }
 
 sub _dump_file {shift->_get_self->_engine->_dump_file;}
 
+sub _warnif {
+ # There is, unfortunately, no way to avoid this hack. warnings.pm does not
+ # allow us to specify exactly the call frame we want. So, for now, we just
+ # look at the bitmask ourselves.
+ my $level;
+ {
+  my($pack, $file, $line, $bitmask) = (caller $level++)[0..2,9];
+  redo if $pack =~ /^DBM::Deep(?:::|\z)/;
+  if(  vec $bitmask, $warnings'Offsets{$_[0]}, 1,
+    || vec $bitmask, $warnings'Offsets{all}, 1,
+    ) {
+     my $msg = $_[1] =~ /\n\z/ ? $_[1] : "$_[1] at $file line $line.\n";
+     die $msg
+      if  vec $bitmask, $warnings'Offsets{$_[0]}+1, 1,
+       || vec $bitmask, $warnings'Offsets{all}+1, 1;
+     warn $msg;
+  }
+ }
+}
+
+sub _free {
+ my $self = shift;
+ if(my $sector = $self->{engine}->load_sector( $self->{base_offset} )) {
+  $sector->free;
+ }
+}
+
+sub DESTROY {
+ my $self = shift;
+ my $alter_ego = $self->_get_self;
+ if( !$alter_ego  ||  $self != $alter_ego ) {
+  return; # Don’t run the destructor twice! (What follows only applies to
+ }        # the inner object, not the tie.)
+
+ # If the engine is gone, the END block has beaten us to it.
+ return if !$self->{engine}; 
+ if(  $self->{engine}->{external_refs} ) {
+  $self->_free;
+ }
+}
+
+# Relying on the destructor alone is problematic, as the order in which
+# objects are discarded is random in global destruction. So we do the
+# clean-up here before preemptively before global destruction.
+END {
+ defined $$_ and  $$_->_free, delete $$_->{engine}
+   for(values %obj_cache);
+}
+
 1;
 __END__
