@@ -1,6 +1,6 @@
 package DBM::Deep::Engine::DBI;
 
-use 5.006_000;
+use 5.008_000;
 
 use strict;
 use warnings FATAL => 'all';
@@ -23,6 +23,7 @@ sub new {
 
     my $self = bless {
         storage => undef,
+        external_refs => undef,
     }, $class;
 
     # Grab the parameters we want to use
@@ -216,7 +217,9 @@ sub write_value {
         or die "Cannot load sector at '@{[$obj->_base_offset]}'\n";;
 
     my ($type, $class);
-    if ( $r eq 'ARRAY' || $r eq 'HASH' ) {
+    if (
+     $r eq 'ARRAY' || $r eq 'HASH' and ref $value ne 'DBM::Deep::Null'
+    ) {
         my $tmpvar;
         if ( $r eq 'ARRAY' ) {
             $tmpvar = tied @$value;
@@ -261,6 +264,13 @@ sub write_value {
             DBM::Deep->_throw_error( "Cannot store something that is tied." );
         }
 
+        if ( ref $value eq 'DBM::Deep::Null' ) {
+            DBM::Deep::_warnif(
+             'uninitialized', 'Assignment of stale reference'
+            );
+            $value = undef;
+        }
+
         $class = 'DBM::Deep::Sector::DBI::Scalar';
         $type  = 'S';
     }
@@ -284,59 +294,59 @@ sub write_value {
     return 1;
 }
 
-sub begin_work {
-    my $self = shift;
-    die "Transactions are not supported by this engine"
-        unless $self->supports('transactions');
-
-    if ( $self->in_txn ) {
-        DBM::Deep->_throw_error( "Cannot begin_work within an active transaction" );
-    }
-
-    $self->storage->begin_work;
-
-    $self->in_txn( 1 );
-
-    return 1;
-} 
-
-sub rollback {
-    my $self = shift;
-    die "Transactions are not supported by this engine"
-        unless $self->supports('transactions');
-
-    if ( !$self->in_txn ) {
-        DBM::Deep->_throw_error( "Cannot rollback without an active transaction" );
-    }
-
-    $self->storage->rollback;
-
-    $self->in_txn( 0 );
-
-    return 1;
-} 
-
-sub commit {
-    my $self = shift;
-    die "Transactions are not supported by this engine"
-        unless $self->supports('transactions');
-
-    if ( !$self->in_txn ) {
-        DBM::Deep->_throw_error( "Cannot commit without an active transaction" );
-    }
-
-    $self->storage->commit;
-
-    $self->in_txn( 0 );
-
-    return 1;
-}
-
-sub in_txn {
-    my $self = shift;
-    $self->{in_txn} = shift if @_;
-    $self->{in_txn};
-}
+#sub begin_work {
+#    my $self = shift;
+#    die "Transactions are not supported by this engine"
+#        unless $self->supports('transactions');
+#
+#    if ( $self->in_txn ) {
+#        DBM::Deep->_throw_error( "Cannot begin_work within an active transaction" );
+#    }
+#
+#    $self->storage->begin_work;
+#
+#    $self->in_txn( 1 );
+#
+#    return 1;
+#} 
+#
+#sub rollback {
+#    my $self = shift;
+#    die "Transactions are not supported by this engine"
+#        unless $self->supports('transactions');
+#
+#    if ( !$self->in_txn ) {
+#        DBM::Deep->_throw_error( "Cannot rollback without an active transaction" );
+#    }
+#
+#    $self->storage->rollback;
+#
+#    $self->in_txn( 0 );
+#
+#    return 1;
+#} 
+#
+#sub commit {
+#    my $self = shift;
+#    die "Transactions are not supported by this engine"
+#        unless $self->supports('transactions');
+#
+#    if ( !$self->in_txn ) {
+#        DBM::Deep->_throw_error( "Cannot commit without an active transaction" );
+#    }
+#
+#    $self->storage->commit;
+#
+#    $self->in_txn( 0 );
+#
+#    return 1;
+#}
+#
+#sub in_txn {
+#    my $self = shift;
+#    $self->{in_txn} = shift if @_;
+#    $self->{in_txn};
+#}
 
 sub supports {
     my $self = shift;
@@ -347,6 +357,10 @@ sub supports {
     return;
 }
 
+sub db_version {
+    return '1.0020'
+}
+
 sub clear {
     my $self = shift;
     my $obj = shift;
