@@ -1,6 +1,6 @@
 package DBM::Deep::Engine::File;
 
-use 5.006_000;
+use 5.008_000;
 
 use strict;
 use warnings FATAL => 'all';
@@ -24,6 +24,7 @@ sub SIG_FILE     () { 'DPDB' }
 sub SIG_HEADER   () { 'h'    }
 sub SIG_NULL     () { 'N'    }
 sub SIG_DATA     () { 'D'    }
+sub SIG_UNIDATA  () { 'U'    }
 sub SIG_INDEX    () { 'I'    }
 sub SIG_BLIST    () { 'B'    }
 sub SIG_FREE     () { 'F'    }
@@ -76,6 +77,8 @@ sub new {
 
         entries => {}, # This is the list of entries for transactions
         storage => undef,
+
+        external_refs => undef,
     }, $class;
 
     # Never allow byte_size to be set directly.
@@ -298,6 +301,13 @@ sub write_value {
     if ( !defined $value ) {
         $class = 'DBM::Deep::Sector::File::Null';
     }
+    elsif ( ref $value eq 'DBM::Deep::Null' ) {
+        DBM::Deep::_warnif(
+             'uninitialized', 'Assignment of stale reference'
+        );
+        $class = 'DBM::Deep::Sector::File::Null';
+        $value = undef;
+    }
     elsif ( $r eq 'ARRAY' || $r eq 'HASH' ) {
         my $tmpvar;
         if ( $r eq 'ARRAY' ) {
@@ -590,7 +600,7 @@ sub get_running_txn_ids {
 =head2 get_txn_staleness_counter( $trans_id )
 
 This will return the staleness counter for the given transaction ID. Please see
-L</TRANSACTION STALENESS> for more information.
+L<DBM::Deep::Engine/STALENESS> for more information.
 
 =cut
 
@@ -612,7 +622,7 @@ sub get_txn_staleness_counter {
 =head2 inc_txn_staleness_counter( $trans_id )
 
 This will increment the staleness counter for the given transaction ID. Please see
-L</TRANSACTION STALENESS> for more information.
+L<DBM::Deep::Engine/STALENESS> for more information.
 
 =cut
 
@@ -714,7 +724,8 @@ settings that set how the file is interpreted.
 
 {
     my $header_fixed = length( __PACKAGE__->SIG_FILE ) + 1 + 4 + 4;
-    my $this_file_version = 3;
+    my $this_file_version = 4;
+    my $min_file_version = 3;
 
     sub _write_file_header {
         my $self = shift;
@@ -750,6 +761,8 @@ settings that set how the file is interpreted.
         $self->set_trans_loc( $header_fixed + 4 );
         $self->set_chains_loc( $header_fixed + 4 + $bl + $STALE_SIZE * ($nt-1) );
 
+        $self->{v} = $this_file_version;
+
         return;
     }
 
@@ -773,13 +786,25 @@ settings that set how the file is interpreted.
             DBM::Deep->_throw_error( "Pre-1.00 file version found" );
         }
 
-        unless ( $file_version == $this_file_version ) {
+        if ( $file_version < $min_file_version ) {
             $self->storage->close;
             DBM::Deep->_throw_error(
-                "Wrong file version found - " .  $file_version .
-                " - expected " . $this_file_version
+                "This file version is too old - "
+              . _guess_version($file_version) .
+                " - expected " . _guess_version($min_file_version)
+              . " to " . _guess_version($this_file_version)
             );
         }
+        if ( $file_version > $this_file_version ) {
+            $self->storage->close;
+            DBM::Deep->_throw_error(
+                "This file version is too new - probably "
+              . _guess_version($file_version) .
+                " - expected " . _guess_version($min_file_version)
+              . " to " . _guess_version($this_file_version)
+            );
+        }
+        $self->{v} = $file_version;
 
         my $buffer2 = $self->storage->read_at( undef, $size );
         my @values = unpack( 'C C C C', $buffer2 );
@@ -809,6 +834,16 @@ settings that set how the file is interpreted.
 
         return length($buffer) + length($buffer2);
     }
+
+    sub _guess_version {
+        $_[0] == 4 and return  2;
+        $_[0] == 3 and return '1.0003';
+        $_[0] == 2 and return '1.00';
+        $_[0] == 1 and return '0.99';
+        $_[0] == 0 and return '0.91';
+
+        return $_[0]-2;
+    }
 }
 
 =head2 _apply_digest( @stuff )
@@ -820,7 +855,9 @@ passed in and return the result.
 
 sub _apply_digest {
     my $self = shift;
-    return $self->{digest}->(@_);
+    my $victim = shift;
+    utf8::encode $victim if $self->{v} >= 4;
+    return $self->{digest}->($victim);
 }
 
 =head2 _add_free_blist_sector( $offset, $size )
@@ -1006,11 +1043,14 @@ sub supports {
     shift;
     my ($feature) = @_;
 
-    return 1 if $feature eq 'transactions';
-    return if $feature eq 'singletones';
+    return 1 if $feature =~ /^(?:(?:transacti|singlet)ons|unicode)\z/;
     return;
 }
 
+sub db_version {
+    return $_[0]{v} == 3 ? '1.0003' : 2;
+}
+
 sub clear {
     my $self = shift;
     my $obj = shift;
@@ -1097,7 +1137,7 @@ sub _dump_file {
         }
         else {
             $return .= sprintf "%08d: %s  %04d", $spot, $sector->type, $sector->size;
-            if ( $sector->type eq 'D' ) {
+            if ( $sector->type =~ /^[DU]\z/ ) {
                 $return .= ' ' . $sector->data;
             }
             elsif ( $sector->type eq 'A' || $sector->type eq 'H' ) {
